@@ -56,6 +56,12 @@ Bool_t TObject::fgObjectStat = kTRUE;
 
 ClassImp(TObject);
 
+#if defined(__clang__) || defined (__GNUC__)
+# define ATTRIBUTE_NO_SANITIZE_ADDRESS __attribute__((no_sanitize_address))
+#else
+# define ATTRIBUTE_NO_SANITIZE_ADDRESS
+#endif
+
 namespace ROOT {
 namespace Internal {
 
@@ -66,11 +72,12 @@ namespace Internal {
 // like TClonesArray, where we know the destructor will be called but not operator
 // delete, we can still use it to detect the cases where the destructor was called.
 
+ATTRIBUTE_NO_SANITIZE_ADDRESS
 bool DeleteChangesMemoryImpl()
 {
    static constexpr UInt_t kGoldenUUID = 0x00000021;
    static constexpr UInt_t kGoldenbits = 0x03000000;
- 
+
    TObject *o = new TObject;
    o->SetUniqueID(kGoldenUUID);
    UInt_t *o_fuid = &(o->fUniqueID);
@@ -285,28 +292,31 @@ void TObject::DrawClass() const
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-/// Draw a clone of this object in the current selected pad for instance with:
-/// `gROOT->SetSelectedPad(gPad)`.
+/// Draw a clone of this object in the current selected pad with:
+/// `gROOT->SetSelectedPad(c1)`.
+/// If pad was not selected - `gPad` will be used.
 
 TObject *TObject::DrawClone(Option_t *option) const
 {
-   TVirtualPad *pad    = gROOT->GetSelectedPad();
-   TVirtualPad *padsav = gPad;
-   if (pad) pad->cd();
+   TVirtualPad::TContext ctxt(true);
+   auto pad = gROOT->GetSelectedPad();
+   if (pad)
+      pad->cd();
 
    TObject *newobj = Clone();
-   if (!newobj) return nullptr;
+   if (!newobj)
+      return nullptr;
+
+   if (!option || !*option)
+      option = GetDrawOption();
+
    if (pad) {
-      if (strlen(option)) pad->GetListOfPrimitives()->Add(newobj,option);
-      else                pad->GetListOfPrimitives()->Add(newobj,GetDrawOption());
+      pad->GetListOfPrimitives()->Add(newobj, option);
       pad->Modified(kTRUE);
       pad->Update();
-      if (padsav) padsav->cd();
-      return newobj;
+   } else {
+      newobj->Draw(option);
    }
-   if (strlen(option))  newobj->Draw(option);
-   else                 newobj->Draw(GetDrawOption());
-   if (padsav) padsav->cd();
 
    return newobj;
 }
@@ -415,9 +425,9 @@ Option_t *TObject::GetDrawOption() const
    if (!gPad) return "";
 
    TListIter next(gPad->GetListOfPrimitives());
-   TObject *obj;
-   while ((obj = next())) {
-      if (obj == this) return next.GetOption();
+   while (auto obj = next()) {
+      if (obj == this)
+         return next.GetOption();
    }
    return "";
 }
@@ -573,8 +583,14 @@ void TObject::ls(Option_t *option) const
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-/// This method must be overridden to handle object notification.
-
+/// This method must be overridden to handle object notification (the base implementation is no-op).
+///
+/// Different objects in ROOT use the `Notify` method for different purposes, in coordination
+/// with other objects that call this method at the appropriate time.
+///
+/// For example, `TLeaf` uses it to load class information; `TBranchRef` to load contents of
+/// referenced branches `TBranchRef`; most notably, based on `Notify`, `TChain` implements a
+/// callback mechanism to inform interested parties when it switches to a new sub-tree.
 Bool_t TObject::Notify()
 {
    return kFALSE;
@@ -604,14 +620,12 @@ void TObject::Pop()
    if (this == gPad->GetListOfPrimitives()->Last()) return;
 
    TListIter next(gPad->GetListOfPrimitives());
-   TObject *obj;
-   while ((obj = next()))
+   while (auto obj = next())
       if (obj == this) {
-         char *opt = StrDup(next.GetOption());
+         TString opt = next.GetOption();
          gPad->GetListOfPrimitives()->Remove((TObject*)this);
-         gPad->GetListOfPrimitives()->AddLast(this, opt);
+         gPad->GetListOfPrimitives()->AddLast(this, opt.Data());
          gPad->Modified();
-         delete [] opt;
          return;
       }
 }
@@ -633,8 +647,9 @@ void TObject::Print(Option_t *) const
 
 Int_t TObject::Read(const char *name)
 {
-   if (gDirectory) return gDirectory->ReadTObject(this,name);
-   else            return 0;
+   if (gDirectory)
+      return gDirectory->ReadTObject(this,name);
+   return 0;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -752,8 +767,7 @@ void TObject::SetDrawOption(Option_t *option)
 
    TListIter next(gPad->GetListOfPrimitives());
    delete gPad->FindObject("Tframe");
-   TObject *obj;
-   while ((obj = next()))
+   while (auto obj = next())
       if (obj == this) {
          next.SetOption(option);
          return;
@@ -851,14 +865,12 @@ Int_t TObject::Write(const char *name, Int_t option, Int_t bufsize) const
    if (option & kOverwrite)   opt += "OverWrite";
    if (option & kWriteDelete) opt += "WriteDelete";
 
-   if (gDirectory) return gDirectory->WriteTObject(this,name,opt.Data(),bufsize);
-   else {
-      const char *objname = "no name specified";
-      if (name) objname = name;
-      else objname = GetName();
-      Error("Write","The current directory (gDirectory) is null. The object (%s) has not been written.",objname);
-      return 0;
-   }
+   if (gDirectory)
+      return gDirectory->WriteTObject(this,name,opt.Data(),bufsize);
+
+   const char *objname = name ? name : GetName();
+   Error("Write","The current directory (gDirectory) is null. The object (%s) has not been written.",objname);
+   return 0;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -901,14 +913,25 @@ void TObject::Streamer(TBuffer &R__b)
       }
    } else {
       R__b.WriteVersion(TObject::IsA());
+      // Can not read TFile.h here and avoid going through the interpreter by
+      // simply hard-coding this value.
+      // This **must** be equal to TFile::k630forwardCompatibility
+      constexpr int TFile__k630forwardCompatibility = BIT(2);
+      const auto parent = R__b.GetParent();
       if (!TestBit(kIsReferenced)) {
          R__b << fUniqueID;
-         R__b << fBits;
+         if (R__unlikely(parent && parent->TestBit(TFile__k630forwardCompatibility)))
+            R__b << fBits;
+         else
+            R__b << (fBits & (~kIsOnHeap & ~kNotDeleted));
       } else {
          //if the object is referenced, we must save its address/file_pid
          UInt_t uid = fUniqueID & 0xffffff;
          R__b << uid;
-         R__b << fBits;
+         if (R__unlikely(parent && parent->TestBit(TFile__k630forwardCompatibility)))
+            R__b << fBits;
+         else
+            R__b << (fBits & (~kIsOnHeap & ~kNotDeleted));
          TProcessID *pid = TProcessID::GetProcessWithUID(fUniqueID,this);
          //add uid to the TRefTable if there is one
          TRefTable *table = TRefTable::GetRefTable();
@@ -1085,7 +1108,6 @@ void TObject::operator delete[](void *ptr)
       fgDtorOnly = 0;
 }
 
-#ifdef R__SIZEDDELETE
 ////////////////////////////////////////////////////////////////////////////////
 /// Operator delete for sized deallocation.
 
@@ -1107,18 +1129,17 @@ void TObject::operator delete[](void *ptr, size_t size)
    else
       fgDtorOnly = 0;
 }
-#endif
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Print value overload
 
-std::string cling::printValue(TObject *val) {
+std::string cling::printValue(TObject *val)
+{
    std::ostringstream strm;
    strm << "Name: " << val->GetName() << " Title: " << val->GetTitle();
    return strm.str();
 }
 
-#ifdef R__PLACEMENTDELETE
 ////////////////////////////////////////////////////////////////////////////////
 /// Only called by placement new when throwing an exception.
 
@@ -1134,4 +1155,3 @@ void TObject::operator delete[](void *ptr, void *vp)
 {
    TStorage::ObjectDealloc(ptr, vp);
 }
-#endif
