@@ -22,6 +22,7 @@
 #include "TList.h"
 #include "TBaseClass.h"
 
+#include <memory>
 #include <sstream>
 #include <regex>
 
@@ -44,8 +45,8 @@ REveDataItemList::REveDataItemList(const std::string& n, const std::string& t):
       REveDataItemList::DummyItemsChange(collection, ids);
    });
 
-   SetFillImpliedSelectedDelegate([&](REveDataItemList *collection, REveElement::Set_t &impSelSet) {
-      REveDataItemList::DummyFillImpliedSelected(collection, impSelSet);
+   SetFillImpliedSelectedDelegate([&](REveDataItemList *collection, REveElement::Set_t &impSelSet, const std::set<int>& sec_idcs) {
+      REveDataItemList::DummyFillImpliedSelected(collection, impSelSet, sec_idcs);
    });
 
    SetupDefaultColorAndTransparency(REveDataCollection::fgDefaultColor, true, true);
@@ -96,14 +97,11 @@ void REveDataItemList::ItemChanged(Int_t idx)
 
 //______________________________________________________________________________
 
-void REveDataItemList::FillImpliedSelectedSet( Set_t& impSelSet)
+void REveDataItemList::FillImpliedSelectedSet(Set_t &impSelSet, const std::set<int> &sec_idcs)
 {
-   /*
-   printf("REveDataItemList::FillImpliedSelectedSet colecction setsize %zu\n",   RefSelectedSet().size());
-   for (auto x : RefSelectedSet())
-      printf("%d \n", x);
-   */
-   fHandlerFillImplied( this ,  impSelSet);
+   // printf("REveDataItemList::FillImpliedSelectedSet colecction setsize %zu\n", sec_idcs.size());
+   RefSelectedSet() = sec_idcs;
+   fHandlerFillImplied(this, impSelSet, sec_idcs);
 }
 
 //______________________________________________________________________________
@@ -158,7 +156,7 @@ void REveDataItemList::ProcessSelectionStr(ElementId_t id, bool multi, bool seco
    try {
       while (itr != end) sis.insert(std::stoi(*itr++));
    }
-   catch (const std::invalid_argument& ia) {
+   catch (const std::invalid_argument&) {
       throw eh + "invalid secondary index argument '" + *itr + "' - must be int.";
    }
 
@@ -210,7 +208,7 @@ std::string REveDataItemList::GetHighlightTooltip(const std::set<int>& secondary
 //______________________________________________________________________________
 void REveDataItemList::AddTooltipExpression(const std::string &title, const std::string &expr, bool init)
 {
-   fTooltipExpressions.push_back(std::unique_ptr<TTip>(new TTip()));
+   fTooltipExpressions.push_back(std::make_unique<TTip>());
    TTip *tt = fTooltipExpressions.back().get();
 
    tt->fTooltipTitle = title;
@@ -247,7 +245,7 @@ void REveDataItemList::DummyItemsChange(REveDataItemList*, const std::vector<int
 
 
 //______________________________________________________________________________
-void REveDataItemList::DummyFillImpliedSelected(REveDataItemList*, REveElement::Set_t&)
+void REveDataItemList::DummyFillImpliedSelected(REveDataItemList*, REveElement::Set_t&, const std::set<int>&)
 {
    if (gDebug) {
       printf("REveDataItemList::DummyFillImpliedSelectedDelegate not implemented\n");
@@ -262,7 +260,7 @@ REveDataCollection::REveDataCollection(const std::string& n, const std::string&
    REveElement(n, t)
 {
    std::string lname = n + "Items";
-   fItemList = new REveDataItemList(lname.c_str());
+   fItemList = new REveDataItemList(lname);
    AddElement(fItemList);
 
    SetupDefaultColorAndTransparency(fgDefaultColor, true, true);
@@ -312,7 +310,7 @@ void REveDataCollection::SetFilterExpr(const char* filter)
          R__LOG_ERROR(REveLog()) << "EveDataCollection::SetFilterExpr" << exc.what();
       }
    }
-   else 
+   else
    {
       // Remove filter
       fFilterFoo = nullptr;
@@ -336,7 +334,7 @@ void REveDataCollection::ApplyFilter()
 {
    if (!fFilterFoo)
       return;
-   
+
    Ids_t ids;
    int idx = 0;
    for (auto &ii : fItemList->fItems)
@@ -358,68 +356,58 @@ void REveDataCollection::ApplyFilter()
 
 void  REveDataCollection::StreamPublicMethods(nlohmann::json &j) const
 {
-   struct PubMethods
-   {
-      void FillJSON(TClass* c, nlohmann::json & arr)
-      {
-         TString  ctor = c->GetName(), dtor = "~";
-         {
-            int i = ctor.Last(':');
-            if (i != kNPOS)
-            {
-               ctor.Replace(0, i + 1, "");
-            }
-            dtor += ctor;
-         }
+   j["fPublicFunctions"] = nlohmann::json::array();
+   TMethod *meth;
+   TIter next(fItemClass->GetListOfAllPublicMethods());
+   int cnt = 0;
+   while ((meth = (TMethod *)next())) {
+      // Filter out ctor, dtor, some ROOT stuff.
 
-         TMethod *meth;
-         TIter    next(c->GetListOfMethods());
-         while ((meth = (TMethod*) next()))
-         {
-            // Filter out ctor, dtor, some ROOT stuff.
-            {
-               TString m(meth->GetName());
-               if (m == ctor || m == dtor ||
-                   m == "Class" || m == "Class_Name" || m == "Class_Version" || m == "Dictionary" || m == "IsA" ||
-                   m == "DeclFileName" || m == "ImplFileName" || m == "DeclFileLine" || m == "ImplFileLine" ||
-                   m == "Streamer" || m == "StreamerNVirtual" || m == "ShowMembers" ||
-                   m == "CheckTObjectHashConsistency")
-               {
-                  continue;
-               }
-            }
+      TString m(meth->GetName());
 
-            TString     ms;
-            TMethodArg *ma;
-            TIter       next_ma(meth->GetListOfMethodArgs());
-            while ((ma = (TMethodArg*) next_ma()))
-            {
-               if ( ! ms.IsNull()) ms += ", ";
-
-               ms += ma->GetTypeName();
-               ms += " ";
-               ms += ma->GetName();
-            }
-            std::string entry(TString::Format("i.%s(%s)",meth->GetName(),ms.Data()).Data());
-            nlohmann::json jm ;
-            jm["f"] = entry;
-            jm["r"] = meth->GetReturnTypeName();
-            jm["c"] = c->GetName();
-            arr.push_back(jm);
-         }
-         {
-            TBaseClass *base;
-            TIter       blnext(c->GetListOfBases());
-            while ((base = (TBaseClass*) blnext()))
-            {
-               FillJSON(base->GetClassPointer(), arr);
-            }
-         }
+      if (m == "Class" || m == "Class_Name" || m == "Class_Version" || m == "Dictionary" || m == "IsA" ||
+          m == "DeclFileName" || m == "ImplFileName" || m == "DeclFileLine" || m == "ImplFileLine" || m == "Streamer" ||
+          m == "StreamerNVirtual" || m == "ShowMembers" || m == "CheckTObjectHashConsistency") {
+         continue;
       }
-   };
-   j["fPublicFunctions"]  = nlohmann::json::array();
-   PubMethods pm;
-   pm.FillJSON(fItemClass, j["fPublicFunctions"]);
+
+      if (m.BeginsWith('~'))
+         continue;
+
+      if (m.Contains("operator"))
+         continue;
+
+      if (meth->GetListOfMethodArgs()->GetLast() > 1)
+         continue;
+
+      if (strcmp(meth->GetReturnTypeName(),"void") == 0)
+         continue;
+
+      TString ms;
+      TMethodArg *ma;
+      TIter next_ma(meth->GetListOfMethodArgs());
+      while ((ma = (TMethodArg *)next_ma())) {
+         if (!ms.IsNull())
+            ms += ", ";
+
+         ms += ma->GetTypeName();
+         ms += " ";
+         ms += ma->GetName();
+      }
+      std::string entry(TString::Format("i.%s(%s)", meth->GetName(), ms.Data()).Data());
+      nlohmann::json jm;
+      jm["f"] = entry;
+      jm["r"] = meth->GetReturnTypeName();
+      jm["c"] = meth->GetClass()->GetName();
+      j["fPublicFunctions"].push_back(jm);
+
+      if (m.Contains("charge"))
+      {
+         printf("FOUND chargw methos %s %d\n\n", m.Data(), cnt);
+      }
+
+      cnt++;
+   }
 }
 
 //______________________________________________________________________________
