@@ -26,6 +26,8 @@
 #include "clang/Sema/Template.h"
 #include "clang/Sema/TemplateDeduction.h"
 
+#include "EnterUserCodeRAII.h"
+
 using namespace clang;
 
 namespace cling {
@@ -320,11 +322,11 @@ namespace cling {
   {
     bool issigned = false;
     bool isunsigned = false;
-    if (typeName.startswith("signed ")) {
+    if (typeName.starts_with("signed ")) {
       issigned = true;
       typeName = StringRef(typeName.data()+7, typeName.size()-7);
     }
-    if (!issigned && typeName.startswith("unsigned ")) {
+    if (!issigned && typeName.starts_with("unsigned ")) {
       isunsigned = true;
       typeName = StringRef(typeName.data()+9, typeName.size()-9);
     }
@@ -386,7 +388,7 @@ namespace cling {
     llvm::StringRef quickTypeName = typeName.trim();
     bool innerConst = false;
     bool outerConst = false;
-    if (quickTypeName.startswith("const ")) {
+    if (quickTypeName.starts_with("const ")) {
       // Use this syntax to avoid the redudant tests in substr.
       quickTypeName = StringRef(quickTypeName.data()+6,
                                 quickTypeName.size()-6);
@@ -395,7 +397,7 @@ namespace cling {
 
     enum PointerType { kPointerType, kLRefType, kRRefType, };
 
-    if (quickTypeName.endswith("const")) {
+    if (quickTypeName.ends_with("const")) {
       if (quickTypeName.size() < 6) return true;
       auto c = quickTypeName[quickTypeName.size()-6];
       if (c==' ' || c=='&' || c=='*') {
@@ -474,6 +476,13 @@ namespace cling {
 
     if (typeName.empty()) return TheQT;
 
+    // findType is called from TClingLookupHelper::GetPartiallyDesugaredNameWithScopeHandling
+    // which is called indirectly from TClassEdit::GetNormalizedName via
+    // the ResolvedTypedef code paths.  Through that code path nothing is
+    // taking the ROOT/Interpreter lock and since this code can modify the
+    // interpreter, we do need to take lock.
+    InterpreterAccessRAII LockAccess(*m_Interpreter);
+
     // Could trigger deserialization of decls.
     Interpreter::PushTransactionRAII RAII(m_Interpreter);
 
@@ -1149,8 +1158,9 @@ namespace cling {
           S.AddMethodCandidate(MD, I.getPair(), MD->getParent(),
                                /*ObjectType=*/ClassType,
                                /*ObjectClassification=*/ObjExprClassification,
-                   llvm::makeArrayRef<Expr*>(GivenArgs.data(), GivenArgs.size()),
-                                   Candidates);
+                               llvm::ArrayRef<Expr*>(GivenArgs.data(),
+                                                     GivenArgs.size()),
+                               Candidates);
         }
         else {
           const FunctionProtoType* Proto = dyn_cast<FunctionProtoType>(
@@ -1164,7 +1174,8 @@ namespace cling {
              continue;
           }
           S.AddOverloadCandidate(FD, I.getPair(),
-                   llvm::makeArrayRef<Expr*>(GivenArgs.data(), GivenArgs.size()),
+                                 llvm::ArrayRef<Expr*>(GivenArgs.data(),
+                                                       GivenArgs.size()),
                                  Candidates);
         }
       }
@@ -1175,19 +1186,20 @@ namespace cling {
             !isa<CXXConstructorDecl>(FTD->getTemplatedDecl())) {
           // Class method template, not static, not a constructor, so has
           // an implicit object argument.
-          S.AddMethodTemplateCandidate(FTD, I.getPair(),
-                                      cast<CXXRecordDecl>(FTD->getDeclContext()),
-                         const_cast<TemplateArgumentListInfo*>(FuncTemplateArgs),
-                                       /*ObjectType=*/ClassType,
-                                  /*ObjectClassification=*/ObjExprClassification,
-                   llvm::makeArrayRef<Expr*>(GivenArgs.data(), GivenArgs.size()),
-                                       Candidates);
+          S.AddMethodTemplateCandidate(
+              FTD, I.getPair(), cast<CXXRecordDecl>(FTD->getDeclContext()),
+              const_cast<TemplateArgumentListInfo*>(FuncTemplateArgs),
+              /*ObjectType=*/ClassType,
+              /*ObjectClassification=*/ObjExprClassification,
+              llvm::ArrayRef<Expr*>(GivenArgs.data(), GivenArgs.size()),
+              Candidates);
         }
         else {
-          S.AddTemplateOverloadCandidate(FTD, I.getPair(),
-                const_cast<TemplateArgumentListInfo*>(FuncTemplateArgs),
-                llvm::makeArrayRef<Expr*>(GivenArgs.data(), GivenArgs.size()),
-                Candidates, /*SuppressUserConversions=*/false);
+          S.AddTemplateOverloadCandidate(
+              FTD, I.getPair(),
+              const_cast<TemplateArgumentListInfo*>(FuncTemplateArgs),
+              llvm::ArrayRef<Expr*>(GivenArgs.data(), GivenArgs.size()),
+              Candidates, /*SuppressUserConversions=*/false);
         }
       }
       else {
