@@ -103,7 +103,7 @@ ParamHistFunc::ParamHistFunc(const char* name, const char* title,
   _numBins = GetNumBins( vars );
 
   // Add the parameters (with checking)
-  addVarSet( vars );
+  _dataVars.addTyped<RooRealVar>(vars);
   addParamSet( paramSet );
 }
 
@@ -138,7 +138,7 @@ ParamHistFunc::ParamHistFunc(const char* name, const char* title,
   _numBins = GetNumBins( vars );
 
   // Add the parameters (with checking)
-  addVarSet( vars );
+  _dataVars.addTyped<RooRealVar>(vars);
   addParamSet( paramSet );
 }
 
@@ -208,7 +208,11 @@ RooAbsReal& ParamHistFunc::getParameter( Int_t index ) const {
   const int j = tmp / n.z;
   const int k = tmp % n.z;
 
-  return static_cast<RooAbsReal&>(_paramSet[i + j * n.x + k * n.xy]);
+  const int idx = i + j * n.x + k * n.xy;
+  if (idx >= _numBins) {
+    throw std::runtime_error("invalid index");
+  }
+  return static_cast<RooAbsReal &>(_paramSet[idx]);
 }
 
 
@@ -289,7 +293,7 @@ RooArgList ParamHistFunc::createParamSet(RooWorkspace& w, const std::string& Pre
 
   RooArgList paramSet;
 
-  Int_t numVars = vars.getSize();
+  Int_t numVars = vars.size();
   Int_t numBins = GetNumBins( vars );
 
   if( numVars == 0 ) {
@@ -327,8 +331,8 @@ RooArgList ParamHistFunc::createParamSet(RooWorkspace& w, const std::string& Pre
     // all starting at 0
     std::vector< Int_t > Indices(numVars, 0);
 
-    RooRealVar* varx = (RooRealVar*) vars.at(0);
-    RooRealVar* vary = (RooRealVar*) vars.at(1);
+    RooRealVar* varx = static_cast<RooRealVar*>(vars.at(0));
+    RooRealVar* vary = static_cast<RooRealVar*>(vars.at(1));
 
     // For each bin, create a RooRealVar
     for( Int_t j = 0; j < vary->numBins(); ++j) {
@@ -362,9 +366,9 @@ RooArgList ParamHistFunc::createParamSet(RooWorkspace& w, const std::string& Pre
     // all starting at 0
     std::vector< Int_t > Indices(numVars, 0);
 
-    RooRealVar* varx = (RooRealVar*) vars.at(0);
-    RooRealVar* vary = (RooRealVar*) vars.at(1);
-    RooRealVar* varz = (RooRealVar*) vars.at(2);
+    RooRealVar* varx = static_cast<RooRealVar*>(vars.at(0));
+    RooRealVar* vary = static_cast<RooRealVar*>(vars.at(1));
+    RooRealVar* varz = static_cast<RooRealVar*>(vars.at(2));
 
     // For each bin, create a RooRealVar
     for( Int_t k = 0; k < varz->numBins(); ++k) {
@@ -478,10 +482,10 @@ RooArgList ParamHistFunc::createParamSet(const std::string& Prefix, Int_t numBin
     VarNameStream << Prefix << "_bin_" << i;
     std::string VarName = VarNameStream.str();
 
-    RooRealVar* gamma = new RooRealVar( VarName.c_str(), VarName.c_str(),
-               gamma_nominal, gamma_min, gamma_max );
+    auto gamma = std::make_unique<RooRealVar>(VarName.c_str(), VarName.c_str(),
+               gamma_nominal, gamma_min, gamma_max);
     gamma->setConstant( false );
-    paramSet.add( *gamma );
+    paramSet.addOwned(std::move(gamma));
 
   }
 
@@ -515,27 +519,6 @@ Int_t ParamHistFunc::getCurrentBin() const {
 }
 
 
-////////////////////////////////////////////////////////////////////////////////
-/// return 0 for success
-/// return 1 for failure
-/// Check that the elements
-/// are actually RooRealVar's
-/// If so, add them to the
-/// list of vars
-Int_t ParamHistFunc::addVarSet( const RooArgList& vars ) {
-  for(auto const& comp : vars) {
-    if (!dynamic_cast<RooRealVar*>(comp)) {
-      auto errorMsg = std::string("ParamHistFunc::(") + GetName() + ") ERROR: component "
-                      + comp->GetName() + " in variables list is not of type RooRealVar";
-      coutE(InputArguments) <<  errorMsg << std::endl;
-      throw std::runtime_error(errorMsg);
-    }
-    _dataVars.add( *comp );
-  }
-  return 0;
-}
-
-
 ////////////////////////////////////////////////////////////////////////////////
 
 Int_t ParamHistFunc::addParamSet( const RooArgList& params ) {
@@ -546,7 +529,7 @@ Int_t ParamHistFunc::addParamSet( const RooArgList& params ) {
   // the right number of arguments:
 
   Int_t numVarBins  = GetNumBins(_dataVars);
-  Int_t numElements = params.getSize();
+  Int_t numElements = params.size();
 
   if( numVarBins != numElements ) {
     std::cout << "ParamHistFunc::addParamSet - ERROR - "
@@ -563,16 +546,7 @@ Int_t ParamHistFunc::addParamSet( const RooArgList& params ) {
   // If so, add them to the
   // list of params
 
-  for (const auto comp : params) {
-    if (!dynamic_cast<const RooAbsReal*>(comp)) {
-      auto errorMsg = std::string("ParamHistFunc::(") + GetName() + ") ERROR: component "
-                      + comp->GetName() + " in parameter list is not of type RooAbsReal.";
-      coutE(InputArguments) <<  errorMsg << std::endl;
-      throw std::runtime_error(errorMsg);
-    }
-
-    _paramSet.add( *comp );
-  }
+  _paramSet.addTyped<RooAbsReal>(params);
 
   return 0;
 }
@@ -586,13 +560,30 @@ double ParamHistFunc::evaluate() const
   return getParameter().getVal();
 }
 
+void ParamHistFunc::translate(RooFit::Detail::CodeSquashContext &ctx) const
+{
+   auto const &n = _numBinsPerDim;
+
+   // check if _numBins needs to be filled
+   if (n.x == 0) {
+      _numBinsPerDim = getNumBinsPerDim(_dataVars);
+   }
+
+   std::string const &idx = _dataSet.calculateTreeIndexForCodeSquash(this, ctx, _dataVars, true);
+   std::string const &paramNames = ctx.buildArg(_paramSet);
+
+   ctx.addResult(this, paramNames + "[" + idx + "]");
+}
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Find all bins corresponding to the values of the observables in `evalData`, and evaluate
 /// the associated parameters.
 /// \param[in,out] evalData Input/output data for evaluating the ParamHistFunc.
 /// \param[in] normSet Normalisation set passed on to objects that are serving values to us.
-void ParamHistFunc::computeBatch(cudaStream_t*, double* output, size_t size, RooFit::Detail::DataMap const& dataMap) const {
+void ParamHistFunc::doEval(RooFit::EvalContext & ctx) const
+{
+  std::span<double> output = ctx.output();
+  std::size_t size = output.size();
 
   auto const& n = _numBinsPerDim;
   // check if _numBins needs to be filled
@@ -608,14 +599,14 @@ void ParamHistFunc::computeBatch(cudaStream_t*, double* output, size_t size, Roo
   // As a working buffer for the bin indices, we use the tail of the output
   // buffer. We can't use the same starting pointer, otherwise we would
   // overwrite the later bin indices as we fill the output.
-  auto indexBuffer = reinterpret_cast<int*>(output + size) - size;
+  auto indexBuffer = reinterpret_cast<int*>(output.data() + size) - size;
   std::fill(indexBuffer, indexBuffer + size, 0); // output buffer for bin indices needs to be zero-initialized
 
   // Use the vectorized RooAbsBinning::binNumbers() to update the total bin
   // index for each dimension, using the `coef` parameter to multiply with the
   // right index multiplication factor for each dimension.
   for (std::size_t iVar = 0; iVar < _dataVars.size(); ++iVar) {
-    _dataSet.getBinnings()[iVar]->binNumbers(dataMap.at(&_dataVars[iVar]).data(), indexBuffer, size, idxMult[iVar]);
+    _dataSet.getBinnings()[iVar]->binNumbers(ctx.at(&_dataVars[iVar]).data(), indexBuffer, size, idxMult[iVar]);
   }
 
   // Finally, look up the parameters and get their values to fill the output buffer
@@ -641,7 +632,7 @@ Int_t ParamHistFunc::getAnalyticalIntegralWN(RooArgSet& allVars, RooArgSet& anal
 
   // Check if this configuration was created before
   Int_t sterileIdx(-1) ;
-  CacheElem* cache = (CacheElem*) _normIntMgr.getObj(normSet,&analVars,&sterileIdx,(const char*)0) ;
+  CacheElem* cache = static_cast<CacheElem*>(_normIntMgr.getObj(normSet,&analVars,&sterileIdx,(const char*)nullptr)) ;
   if (cache) {
     return _normIntMgr.lastIndex()+1 ;
   }
@@ -650,7 +641,7 @@ Int_t ParamHistFunc::getAnalyticalIntegralWN(RooArgSet& allVars, RooArgSet& anal
   cache = new CacheElem ;
 
   // Store cache element
-  Int_t code = _normIntMgr.setObj(normSet,&analVars,(RooAbsCacheElement*)cache,0) ;
+  Int_t code = _normIntMgr.setObj(normSet,&analVars,cache,nullptr) ;
 
   return code+1 ;
 
@@ -704,7 +695,7 @@ std::list<double>* ParamHistFunc::plotSamplingHint(RooAbsRealLValue& obs, double
 
   std::list<double>* hint = new std::list<double> ;
 
-  // Widen range slighty
+  // Widen range slightly
   xlo = xlo - 0.01*(xhi-xlo) ;
   xhi = xhi + 0.01*(xhi-xlo) ;
 
@@ -727,25 +718,28 @@ std::list<double>* ParamHistFunc::plotSamplingHint(RooAbsRealLValue& obs, double
 /// as the recursive division strategy of RooCurve cannot deal efficiently
 /// with the vertical lines that occur in a non-interpolated histogram
 
-std::list<double>* ParamHistFunc::binBoundaries(RooAbsRealLValue& obs, double xlo,
-                    double xhi) const
+std::list<double> *ParamHistFunc::binBoundaries(RooAbsRealLValue &obs, double xlo, double xhi) const
 {
-  // copied and edited from RooHistFunc
-  RooAbsLValue* lvarg = &obs;
+   // copied and edited from RooHistFunc
+   RooAbsLValue *lvarg = &obs;
 
-  // Retrieve position of all bin boundaries
-  const RooAbsBinning* binning = lvarg->getBinningPtr(nullptr);
-  double* boundaries = binning->array() ;
-
-  std::list<double>* hint = new std::list<double> ;
-
-  // Construct array with pairs of points positioned epsilon to the left and
-  // right of the bin boundaries
-  for (Int_t i=0 ; i<binning->numBoundaries() ; i++) {
-    if (boundaries[i]>=xlo && boundaries[i]<=xhi) {
-      hint->push_back(boundaries[i]) ;
-    }
-  }
-
-  return hint ;
+   // look for variable in the DataHist, and if found, return the binning
+   std::string varName = dynamic_cast<TObject *>(lvarg)->GetName();
+   RooArgSet const &vars = *_dataSet.get(); // guaranteed to be in the same order as the binnings vector
+   auto &binnings = _dataSet.getBinnings();
+   for (size_t i = 0; i < vars.size(); i++) {
+      if (varName == vars[i]->GetName()) {
+         // found the variable, return its binning
+         double *boundaries = binnings.at(i)->array();
+         std::list<double> *hint = new std::list<double>;
+         for (int j = 0; j < binnings.at(i)->numBoundaries(); j++) {
+            if (boundaries[j] >= xlo && boundaries[j] <= xhi) {
+               hint->push_back(boundaries[j]);
+            }
+         }
+         return hint;
+      }
+   }
+   // variable not found, return null
+   return nullptr;
 }
