@@ -37,19 +37,19 @@
 #include <fstream>
 #include <iomanip>
 
-using namespace std;
+using std::endl, std::ofstream, std::cout;
 
 RooAbsMinimizerFcn::RooAbsMinimizerFcn(RooArgList paramList, RooMinimizer *context) : _context(context)
 {
    // Examine parameter list
-   _floatParamList.reset((RooArgList *)paramList.selectByAttrib("Constant", false));
-   if (_floatParamList->getSize() > 1) {
+   _floatParamList.reset(static_cast<RooArgList *>(paramList.selectByAttrib("Constant", false)));
+   if (_floatParamList->size() > 1) {
       _floatParamList->sort();
    }
    _floatParamList->setName("floatParamList");
 
-   _constParamList.reset((RooArgList *)paramList.selectByAttrib("Constant", true));
-   if (_constParamList->getSize() > 1) {
+   _constParamList.reset(static_cast<RooArgList *>(paramList.selectByAttrib("Constant", true)));
+   if (_constParamList->size() > 1) {
       _constParamList->sort();
    }
    _constParamList->setName("constParamList");
@@ -66,20 +66,31 @@ RooAbsMinimizerFcn::RooAbsMinimizerFcn(RooArgList paramList, RooMinimizer *conte
       }
    }
 
-   _nDim = _floatParamList->getSize();
+   _nDim = _floatParamList->size();
 
    // Save snapshot of initial lists
-   _initFloatParamList.reset((RooArgList *)_floatParamList->snapshot(false));
-   _initConstParamList.reset((RooArgList *)_constParamList->snapshot(false));
+   _initFloatParamList = std::make_unique<RooArgList>();
+   _initConstParamList = std::make_unique<RooArgList>();
+   _floatParamList->snapshot(*_initFloatParamList, false);
+   _constParamList->snapshot(*_initConstParamList, false);
 }
 
 RooAbsMinimizerFcn::RooAbsMinimizerFcn(const RooAbsMinimizerFcn &other)
-   : _context(other._context), _maxFCN(other._maxFCN), _funcOffset(other._funcOffset), _numBadNLL(other._numBadNLL),
-     _evalCounter(other._evalCounter), _nDim(other._nDim), _optConst(other._optConst),
-     _floatParamList(new RooArgList(*other._floatParamList)), _constParamList(new RooArgList(*other._constParamList)),
-     _initFloatParamList((RooArgList *)other._initFloatParamList->snapshot(false)),
-     _initConstParamList((RooArgList *)other._initConstParamList->snapshot(false)), _logfile(other._logfile)
+   : _context(other._context),
+     _maxFCN(other._maxFCN),
+     _funcOffset(other._funcOffset),
+     _numBadNLL(other._numBadNLL),
+     _evalCounter(other._evalCounter),
+     _nDim(other._nDim),
+     _optConst(other._optConst),
+     _floatParamList(std::make_unique<RooArgList>(*other._floatParamList)),
+     _constParamList(std::make_unique<RooArgList>(*other._constParamList)),
+     _initFloatParamList(std::make_unique<RooArgList>()),
+     _initConstParamList(std::make_unique<RooArgList>()),
+     _logfile(other._logfile)
 {
+   other._initFloatParamList->snapshot(*_initFloatParamList, false);
+   other._initConstParamList->snapshot(*_initConstParamList, false);
 }
 
 /// Internal function to synchronize TMinimizer with current
@@ -90,10 +101,8 @@ bool RooAbsMinimizerFcn::synchronizeParameterSettings(std::vector<ROOT::Fit::Par
    bool constValChange(false);
    bool constStatChange(false);
 
-   int index(0);
-
    // Handle eventual migrations from constParamList -> floatParamList
-   for (index = 0; index < _constParamList->getSize(); index++) {
+   for (std::size_t index = 0; index < _constParamList->size(); index++) {
 
       RooRealVar *par = dynamic_cast<RooRealVar *>(_constParamList->at(index));
       if (!par)
@@ -132,11 +141,11 @@ bool RooAbsMinimizerFcn::synchronizeParameterSettings(std::vector<ROOT::Fit::Par
    }
 
    // Update reference list
-   *_initConstParamList = *_constParamList;
+   _initConstParamList->assign(*_constParamList);
 
    // Synchronize MINUIT with function state
    // Handle floatParamList
-   for (index = 0; index < _floatParamList->getSize(); index++) {
+   for (std::size_t index = 0; index < _floatParamList->size(); index++) {
       RooRealVar *par = dynamic_cast<RooRealVar *>(_floatParamList->at(index));
 
       if (!par)
@@ -202,16 +211,17 @@ bool RooAbsMinimizerFcn::synchronizeParameterSettings(std::vector<ROOT::Fit::Par
       }
 
       // new parameter
-      if (index >= int(parameters.size())) {
+      if (index >= parameters.size()) {
 
          if (par->hasMin() && par->hasMax()) {
             parameters.emplace_back(par->GetName(), par->getVal(), pstep, pmin, pmax);
          } else {
             parameters.emplace_back(par->GetName(), par->getVal(), pstep);
-            if (par->hasMin())
+            if (par->hasMin()) {
                parameters.back().SetLowerLimit(pmin);
-            else if (par->hasMax())
+            } else if (par->hasMax()) {
                parameters.back().SetUpperLimit(pmax);
+            }
          }
 
          continue;
@@ -271,12 +281,13 @@ bool RooAbsMinimizerFcn::synchronizeParameterSettings(std::vector<ROOT::Fit::Par
          if (oldVar != par->getVal() || oldVlo != pmin || oldVhi != pmax || oldVerr != pstep) {
             parameters[index].SetValue(par->getVal());
             parameters[index].SetStepSize(pstep);
-            if (par->hasMin() && par->hasMax())
+            if (par->hasMin() && par->hasMax()) {
                parameters[index].SetLimits(pmin, pmax);
-            else if (par->hasMin())
+            } else if (par->hasMin()) {
                parameters[index].SetLowerLimit(pmin);
-            else if (par->hasMax())
+            } else if (par->hasMax()) {
                parameters[index].SetUpperLimit(pmax);
+            }
          }
 
          // Inform user about changes in verbose mode
@@ -308,7 +319,7 @@ bool RooAbsMinimizerFcn::synchronizeParameterSettings(std::vector<ROOT::Fit::Par
       optimizeConstantTerms(constStatChange, constValChange);
    }
 
-   return 0;
+   return false;
 }
 
 bool RooAbsMinimizerFcn::Synchronize(std::vector<ROOT::Fit::ParameterSettings> &parameters)
@@ -367,14 +378,14 @@ bool RooAbsMinimizerFcn::SetLogFile(const char *inLogfile)
       oocoutI(_context, Minimization) << "RooAbsMinimizerFcn::setLogFile: closing previous log file" << endl;
       _logfile->close();
       delete _logfile;
-      _logfile = 0;
+      _logfile = nullptr;
    }
    _logfile = new ofstream(inLogfile);
    if (!_logfile->good()) {
       oocoutI(_context, Minimization) << "RooAbsMinimizerFcn::setLogFile: cannot open file " << inLogfile << endl;
       _logfile->close();
       delete _logfile;
-      _logfile = 0;
+      _logfile = nullptr;
    }
 
    return false;
@@ -438,6 +449,36 @@ void RooAbsMinimizerFcn::printEvalErrors() const
    ooccoutW(_context, Minimization) << msg.str() << endl;
 }
 
+/// Apply corrections on the fvalue if errors were signaled.
+///
+/// Two kinds of errors are possible: 1. infinite or nan values (the latter
+/// can be a signaling nan, using RooNaNPacker) or 2. logEvalError-type errors.
+/// Both are caught here and fvalue is updated so that Minuit in turn is nudged
+/// to move the search outside of the problematic parameter space area.
+double RooAbsMinimizerFcn::applyEvalErrorHandling(double fvalue) const
+{
+   if (!std::isfinite(fvalue) || RooAbsReal::numEvalErrors() > 0 || fvalue > 1e30) {
+      printEvalErrors();
+      RooAbsReal::clearEvalErrorLog();
+      _numBadNLL++;
+
+      if (cfg().doEEWall) {
+         const double badness = RooNaNPacker::unpackNaN(fvalue);
+         fvalue = (std::isfinite(_maxFCN) ? _maxFCN : 0.) + cfg().recoverFromNaN * badness;
+      }
+   } else {
+      if (_evalCounter > 0 && _evalCounter == _numBadNLL) {
+         // This is the first time we get a valid function value; while before, the
+         // function was always invalid. For invalid  cases, we returned values > 0.
+         // Now, we offset valid values such that they are < 0.
+         _funcOffset = -fvalue;
+      }
+      fvalue += _funcOffset;
+      _maxFCN = std::max(fvalue, _maxFCN);
+   }
+   return fvalue;
+}
+
 void RooAbsMinimizerFcn::finishDoEval() const
 {
 
@@ -459,25 +500,29 @@ void RooAbsMinimizerFcn::setOptimizeConst(int flag)
    auto ctx = _context->makeEvalErrorContext();
 
    if (_optConst && !flag) {
-      if (_context->getPrintLevel() > -1)
+      if (_context->getPrintLevel() > -1) {
          oocoutI(_context, Minimization) << "RooAbsMinimizerFcn::setOptimizeConst: deactivating const optimization"
                                          << endl;
+      }
       setOptimizeConstOnFunction(RooAbsArg::DeActivate, true);
       _optConst = flag;
    } else if (!_optConst && flag) {
-      if (_context->getPrintLevel() > -1)
+      if (_context->getPrintLevel() > -1) {
          oocoutI(_context, Minimization) << "RooAbsMinimizerFcn::setOptimizeConst: activating const optimization"
                                          << endl;
+      }
       setOptimizeConstOnFunction(RooAbsArg::Activate, flag > 1);
       _optConst = flag;
    } else if (_optConst && flag) {
-      if (_context->getPrintLevel() > -1)
+      if (_context->getPrintLevel() > -1) {
          oocoutI(_context, Minimization) << "RooAbsMinimizerFcn::setOptimizeConst: const optimization already active"
                                          << endl;
+      }
    } else {
-      if (_context->getPrintLevel() > -1)
+      if (_context->getPrintLevel() > -1) {
          oocoutI(_context, Minimization) << "RooAbsMinimizerFcn::setOptimizeConst: const optimization wasn't active"
                                          << endl;
+      }
    }
 }
 
@@ -506,7 +551,7 @@ std::vector<double> RooAbsMinimizerFcn::getParameterValues() const
    values.reserve(_nDim);
 
    for (std::size_t index = 0; index < _nDim; ++index) {
-      RooRealVar *par = (RooRealVar *)_floatParamList->at(index);
+      RooRealVar *par = static_cast<RooRealVar *>(_floatParamList->at(index));
       values.push_back(par->getVal());
    }
 
