@@ -19,7 +19,7 @@
 \class RooAddGenContext
 \ingroup Roofitcore
 
-RooAddGenContext is an efficient implementation of the
+Efficient implementation of the
 generator context specific for RooAddPdf PDFs. The strategy
 of RooAddGenContext is to defer generation of each component
 to a dedicated generator context for that component and to
@@ -56,8 +56,9 @@ RooAddGenContext::RooAddGenContext(const RooAddPdf &model, const RooArgSet &vars
   ccxcoutI(Generation) << std::endl;
 
   // Constructor. Build an array of generator contexts for each product component PDF
-  _pdfSet.reset(static_cast<RooArgSet*>(RooArgSet(model).snapshot(true)));
-  _pdf = (RooAddPdf*) _pdfSet->find(model.GetName()) ;
+  _pdfSet = std::make_unique<RooArgSet>();
+  RooArgSet(model).snapshot(*_pdfSet, true);
+  _pdf = static_cast<RooAddPdf*>(_pdfSet->find(model.GetName())) ;
   _pdf->setOperMode(RooAbsArg::ADirty,true) ;
 
   // Fix normalization set of this RooAddPdf
@@ -68,9 +69,10 @@ RooAddGenContext::RooAddGenContext(const RooAddPdf &model, const RooArgSet &vars
       _pdf->fixAddCoefNormalization(coefNSet,false) ;
     }
 
-  _nComp = model._pdfList.getSize() ;
+  _nComp = model._pdfList.size() ;
   _coefThresh.resize(_nComp+1);
-  _vars.reset(static_cast<RooArgSet*>(vars.snapshot(false)));
+  _vars = std::make_unique<RooArgSet>();
+  vars.snapshot(*_vars, false);
 
   for (const auto arg : model._pdfList) {
     auto pdf = dynamic_cast<const RooAbsPdf *>(arg);
@@ -83,7 +85,7 @@ RooAddGenContext::RooAddGenContext(const RooAddPdf &model, const RooArgSet &vars
     _gcList.emplace_back(pdf->genContext(vars,prototype,auxProto,verbose));
   }
 
-  ((RooAddPdf*)_pdf)->getProjCache(_vars.get()) ;
+  (static_cast<RooAddPdf*>(_pdf))->getProjCache(_vars.get()) ;
   _pdf->recursiveRedirectServers(_theEvent) ;
 }
 
@@ -104,19 +106,21 @@ RooAddGenContext::RooAddGenContext(const RooAddModel &model, const RooArgSet &va
   ccxcoutI(Generation) << std::endl;
 
   // Constructor. Build an array of generator contexts for each product component PDF
-  _pdfSet.reset(static_cast<RooArgSet*>(RooArgSet(model).snapshot(true)));
-  _pdf = (RooAbsPdf*) _pdfSet->find(model.GetName()) ;
+  _pdfSet = std::make_unique<RooArgSet>();
+  RooArgSet(model).snapshot(*_pdfSet, true);
+  _pdf = static_cast<RooAbsPdf*>(_pdfSet->find(model.GetName())) ;
 
-  _nComp = model._pdfList.getSize() ;
+  _nComp = model._pdfList.size() ;
   _coefThresh.resize(_nComp+1);
-  _vars.reset(static_cast<RooArgSet*>(vars.snapshot(false)));
+  _vars = std::make_unique<RooArgSet>();
+  vars.snapshot(*_vars, false);
 
   for (const auto obj : model._pdfList) {
     auto pdf = static_cast<RooAbsPdf*>(obj);
     _gcList.emplace_back(pdf->genContext(vars,prototype,auxProto,verbose));
   }
 
-  ((RooAddModel*)_pdf)->getProjCache(_vars.get()) ;
+  (static_cast<RooAddModel*>(_pdf))->getProjCache(_vars.get()) ;
   _pdf->recursiveRedirectServers(_theEvent) ;
 }
 
@@ -137,7 +141,7 @@ void RooAddGenContext::attach(const RooArgSet& args)
 
 
 ////////////////////////////////////////////////////////////////////////////////
-/// One-time initialization of generator contex. Attach theEvent
+/// One-time initialization of generator context. Attach theEvent
 /// to internal p.d.f clone and forward initialization call to
 /// the component generators
 
@@ -146,10 +150,10 @@ void RooAddGenContext::initGenerator(const RooArgSet &theEvent)
   _pdf->recursiveRedirectServers(theEvent) ;
 
   if (_isModel) {
-    RooAddModel* amod = (RooAddModel*) _pdf ;
+    RooAddModel* amod = static_cast<RooAddModel*>(_pdf) ;
     _pcache = amod->getProjCache(_vars.get()) ;
   } else {
-    RooAddPdf* apdf = (RooAddPdf*) _pdf ;
+    RooAddPdf* apdf = static_cast<RooAddPdf*>(_pdf) ;
     _pcache = apdf->getProjCache(_vars.get(),nullptr) ;
   }
 
@@ -166,7 +170,7 @@ void RooAddGenContext::initGenerator(const RooArgSet &theEvent)
 
 void RooAddGenContext::generateEvent(RooArgSet &theEvent, Int_t remaining)
 {
-  // Throw a random number to determin which component to generate
+  // Throw a random number to determine which component to generate
   updateThresholds() ;
   double rand = RooRandom::uniform() ;
   for (Int_t i=0 ; i<_nComp ; i++) {
@@ -195,7 +199,7 @@ void RooAddGenContext::updateThresholds()
         std::stringstream errMsgStream;
         errMsgStream << "RooAddGenContext::updateThresholds(): coefficient number " << i << " of the "
                      << pdf->ClassName() << " \"" << pdf->GetName() <<  "\"" << " is negative!"
-                     << " The current RooAddGenConext doesn't support negative coefficients."
+                     << " The current RooAddGenContext doesn't support negative coefficients."
                      << " Please recreate a new generator context with " << pdf->ClassName() << "::genContext()";
         auto const errMsg = errMsgStream.str();
         cxcoutE(Generation) << errMsg << std::endl;
