@@ -46,7 +46,7 @@ RooRealSumFunc::RooRealSumFunc() : _normIntMgr(this, 10)
 {
    // Default constructor
    // coverity[UNINIT_CTOR]
-   TRACE_CREATE
+   TRACE_CREATE;
 }
 
 //_____________________________________________________________________________
@@ -55,7 +55,7 @@ RooRealSumFunc::RooRealSumFunc(const char *name, const char *title)
      _funcList("!funcList", "List of functions", this), _coefList("!coefList", "List of coefficients", this)
 {
    // Constructor with name and title
-   TRACE_CREATE
+   TRACE_CREATE;
 }
 
 //_____________________________________________________________________________
@@ -72,7 +72,7 @@ RooRealSumFunc::RooRealSumFunc(const char *name, const char *title, RooAbsReal &
    _funcList.add(func1);
    _funcList.add(func2);
    _coefList.add(coef1);
-   TRACE_CREATE
+   TRACE_CREATE;
 }
 
 //_____________________________________________________________________________
@@ -88,7 +88,7 @@ RooRealSumFunc::RooRealSumFunc(const char *name, const char *title, const RooArg
 
    RooRealSumPdf::initializeFuncsAndCoefs(*this, inFuncList, inCoefList, _funcList, _coefList);
 
-   TRACE_CREATE
+   TRACE_CREATE;
 }
 
 //_____________________________________________________________________________
@@ -99,13 +99,13 @@ RooRealSumFunc::RooRealSumFunc(const RooRealSumFunc &other, const char *name)
 {
    // Copy constructor
 
-   TRACE_CREATE
+   TRACE_CREATE;
 }
 
 //_____________________________________________________________________________
 RooRealSumFunc::~RooRealSumFunc()
 {
-   TRACE_DESTROY
+   TRACE_DESTROY;
 }
 
 //_____________________________________________________________________________
@@ -116,7 +116,7 @@ double RooRealSumFunc::evaluate() const
 
 void RooRealSumFunc::translate(RooFit::Detail::CodeSquashContext &ctx) const
 {
-   RooRealSumPdf::translateImpl(ctx, this, _funcList, _coefList);
+   ctx.addResult(this, RooRealSumPdf::translateImpl(ctx, this, _funcList, _coefList));
 }
 
 //_____________________________________________________________________________
