@@ -1,3 +1,5 @@
+#include <memory>
+
 #include "TFile.h"
 #include "TTree.h"
 #include "TBranch.h"
@@ -30,7 +32,7 @@ protected:
          "ROOT PCM", false);
    }
 
-   virtual void SetUp()
+   void SetUp() override
    {
       TRandom *random = new TRandom(837);
       auto file = new TFile("TOffsetGeneration1.root", "RECREATE");
@@ -117,9 +119,9 @@ protected:
       sample2.i = 1;
       sample2.d = d;
       auto br = tree->Branch("sample", &sample2, 32*1024, 99);
-      br->SetAutoDelete(kFALSE);
+      br->SetAutoDelete(false);
       auto br2 = tree2->Branch("sample", &sample2, 32*1024, 99);
-      br2->SetAutoDelete(kFALSE);
+      br2->SetAutoDelete(false);
       tree->Branch("elem", &elem, "elem/I");
       tree->Branch("sample2", &sample, "sample2[elem]/I");
 
@@ -177,7 +179,7 @@ TEST_F(TOffsetGeneration, primitiveTest)
    auto br = tree->GetBranch("sample");
    ASSERT_TRUE(br->GetTotalSize() < fEventCount * 14);
 
-   file.reset(new TFile("TOffsetGeneration2.root"));
+   file = std::make_unique<TFile>("TOffsetGeneration2.root");
    tree = static_cast<TTree *>(file->Get("tree"));
    br = tree->GetBranch("sample");
    ASSERT_TRUE(br->GetTotalSize() > fEventCount * 14);
@@ -190,7 +192,7 @@ TEST_F(TOffsetGeneration, elementsTest)
    auto br = tree->GetBranch("d");
    ASSERT_TRUE(br->GetTotalSize() > fEventCount * 10);
 
-   file.reset(new TFile("TOffsetGeneration4.root"));
+   file = std::make_unique<TFile>("TOffsetGeneration4.root");
    tree = static_cast<TTree *>(file->Get("tree2"));
    br = tree->GetBranch("d");
    TClass *expectedClass = nullptr;
