@@ -18,6 +18,7 @@
 #include "TBranchObject.h"
 #include "TBranchProxyDirector.h"
 #include "TClassEdit.h"
+#include "TEnum.h"
 #include "TFriendElement.h"
 #include "TFriendProxy.h"
 #include "TLeaf.h"
@@ -40,59 +41,59 @@ namespace {
    // Reader interface for clones arrays
    class TClonesReader: public TVirtualCollectionReader {
    public:
-      ~TClonesReader() {}
+      ~TClonesReader() override {}
       TClonesArray* GetCA(ROOT::Detail::TBranchProxy* proxy) {
          if (!proxy->Read()){
             fReadStatus = TTreeReaderValueBase::kReadError;
             Error("TClonesReader::GetCA()", "Read error in TBranchProxy.");
-            return 0;
+            return nullptr;
          }
          fReadStatus = TTreeReaderValueBase::kReadSuccess;
          return (TClonesArray*) proxy->GetWhere();
       }
-      virtual size_t GetSize(ROOT::Detail::TBranchProxy* proxy) {
+      size_t GetSize(ROOT::Detail::TBranchProxy* proxy) override {
          TClonesArray *myClonesArray = GetCA(proxy);
          if (myClonesArray){
             return myClonesArray->GetEntries();
          }
          else return 0;
       }
-      virtual void* At(ROOT::Detail::TBranchProxy* proxy, size_t idx) {
+      void* At(ROOT::Detail::TBranchProxy* proxy, size_t idx) override {
          TClonesArray *myClonesArray = GetCA(proxy);
          if (myClonesArray){
             return myClonesArray->UncheckedAt(idx);
          }
-         else return 0;
+         else return nullptr;
       }
    };
 
    // Reader interface for STL
    class TSTLReader final: public TVirtualCollectionReader {
    public:
-      ~TSTLReader() {}
+      ~TSTLReader() override {}
       TVirtualCollectionProxy* GetCP(ROOT::Detail::TBranchProxy* proxy) {
          if (!proxy->Read()) {
             fReadStatus = TTreeReaderValueBase::kReadError;
             Error("TSTLReader::GetCP()", "Read error in TBranchProxy.");
-            return 0;
+            return nullptr;
          }
          if (!proxy->GetWhere()) {
             Error("TSTLReader::GetCP()", "Logic error, proxy object not set in TBranchProxy.");
-            return 0;
+            return nullptr;
          }
          fReadStatus = TTreeReaderValueBase::kReadSuccess;
          return (TVirtualCollectionProxy*) proxy->GetCollection();
       }
 
-      virtual size_t GetSize(ROOT::Detail::TBranchProxy* proxy) {
+      size_t GetSize(ROOT::Detail::TBranchProxy* proxy) override {
          TVirtualCollectionProxy *myCollectionProxy = GetCP(proxy);
          if (!myCollectionProxy) return 0;
          return myCollectionProxy->Size();
       }
 
-      virtual void* At(ROOT::Detail::TBranchProxy* proxy, size_t idx) {
+      void* At(ROOT::Detail::TBranchProxy* proxy, size_t idx) override {
          TVirtualCollectionProxy *myCollectionProxy = GetCP(proxy);
-         if (!myCollectionProxy) return 0;
+         if (!myCollectionProxy) return nullptr;
          if (myCollectionProxy->HasPointers()){
             return *(void**)myCollectionProxy->At(idx);
          }
@@ -112,17 +113,17 @@ namespace {
          if (!proxy->Read()) {
             fReadStatus = TTreeReaderValueBase::kReadError;
             Error("TCollectionLessSTLReader::GetCP()", "Read error in TBranchProxy.");
-            return 0;
+            return nullptr;
          }
          if (!proxy->GetWhere()) {
             Error("TCollectionLessSTLReader::GetCP()", "Logic error, proxy object not set in TBranchProxy.");
-            return 0;
+            return nullptr;
          }
          fReadStatus = TTreeReaderValueBase::kReadSuccess;
          return fLocalCollection;
       }
 
-      virtual size_t GetSize(ROOT::Detail::TBranchProxy* proxy) {
+      size_t GetSize(ROOT::Detail::TBranchProxy* proxy) override {
          TVirtualCollectionProxy *myCollectionProxy = GetCP(proxy);
          if (!myCollectionProxy) return 0;
          /// In the case of std::vector<bool> `PushProxy` also creates a temporary bool variable the address of which
@@ -132,9 +133,9 @@ namespace {
          return myCollectionProxy->Size();
       }
 
-      virtual void* At(ROOT::Detail::TBranchProxy* proxy, size_t idx) {
+      void* At(ROOT::Detail::TBranchProxy* proxy, size_t idx) override {
          TVirtualCollectionProxy *myCollectionProxy = GetCP(proxy);
-         if (!myCollectionProxy) return 0;
+         if (!myCollectionProxy) return nullptr;
          // Here we do not use a RAII but we empty the proxy to then fill it.
          // This is done because we are returning a pointer and we need to keep
          // alive the memory it points to.
@@ -156,23 +157,23 @@ namespace {
       Int_t fBasicTypeSize;
    public:
       TObjectArrayReader() : fBasicTypeSize(-1) { }
-      ~TObjectArrayReader() {}
+      ~TObjectArrayReader() override {}
       TVirtualCollectionProxy* GetCP(ROOT::Detail::TBranchProxy* proxy) {
          if (!proxy->Read()){
             fReadStatus = TTreeReaderValueBase::kReadError;
             Error("TObjectArrayReader::GetCP()", "Read error in TBranchProxy.");
-            return 0;
+            return nullptr;
          }
          fReadStatus = TTreeReaderValueBase::kReadSuccess;
          return (TVirtualCollectionProxy*) proxy->GetCollection();
       }
-      virtual size_t GetSize(ROOT::Detail::TBranchProxy* proxy) {
+      size_t GetSize(ROOT::Detail::TBranchProxy* proxy) override {
          TVirtualCollectionProxy *myCollectionProxy = GetCP(proxy);
          if (!myCollectionProxy) return 0;
          return myCollectionProxy->Size();
       }
-      virtual void* At(ROOT::Detail::TBranchProxy* proxy, size_t idx) {
-         if (!proxy->Read()) return 0;
+      void* At(ROOT::Detail::TBranchProxy* proxy, size_t idx) override {
+         if (!proxy->Read()) return nullptr;
 
          Int_t objectSize;
          void *array = (void*)proxy->GetStart();
@@ -181,7 +182,7 @@ namespace {
             TClass *myClass = proxy->GetClass();
             if (!myClass){
                Error("TObjectArrayReader::At()", "Cannot get class info from branch proxy.");
-               return 0;
+               return nullptr;
             }
             objectSize = myClass->GetClassSize();
          }
@@ -197,23 +198,30 @@ namespace {
    };
 
    template <class BASE>
-   class TUIntOrIntReader: public BASE {
-   private:
-      // The index can be of type int or unsigned int.
-      std::unique_ptr<TTreeReaderValueBase> fSizeReader;
-      bool fIsUnsigned = false;
+   class TDynamicArrayReader : public BASE {
 
-   protected:
-      template <class T>
-      TTreeReaderValue<T>& GetSizeReader() {
-         return *static_cast<TTreeReaderValue<T>*>(fSizeReader.get());
-      }
+      // TVirtualSizeReaderImpl and TSizeReaderImpl type-erase the reading of the size leaf.
+      class TVirtualSizeReaderImpl {
+      public:
+         virtual ~TVirtualSizeReaderImpl() = default;
+         virtual size_t GetSize() = 0;
+      };
+
+      template <typename T>
+      class TSizeReaderImpl final : public TVirtualSizeReaderImpl {
+         TTreeReaderValue<T> fSizeReader;
+
+      public:
+         TSizeReaderImpl(TTreeReader &r, const char *leafName) : fSizeReader(r, leafName) {}
+         size_t GetSize() final { return *fSizeReader; }
+      };
+
+      std::unique_ptr<TVirtualSizeReaderImpl> fSizeReader;
 
    public:
       template <class... ARGS>
-      TUIntOrIntReader(TTreeReader *treeReader, const char *leafName,
-                       ARGS&&... args):
-         BASE(std::forward<ARGS>(args)...)
+      TDynamicArrayReader(TTreeReader *treeReader, const char *leafName, ARGS &&...args)
+         : BASE(std::forward<ARGS>(args)...)
       {
          std::string foundLeafName = leafName;
          TLeaf* sizeLeaf = treeReader->GetTree()->FindLeaf(foundLeafName.c_str());
@@ -247,29 +255,44 @@ namespace {
          }
 
          if (!sizeLeaf) {
-            Error("TUIntOrIntReader", "Cannot find leaf count for %s or any parent branch!", leafName);
+            Error("TDynamicArrayReader ", "Cannot find leaf count for %s or any parent branch!", leafName);
             return;
          }
 
-         fIsUnsigned = sizeLeaf->IsUnsigned();
-         if (fIsUnsigned) {
-            fSizeReader.reset(new TTreeReaderValue<UInt_t>(*treeReader, foundLeafName.c_str()));
+         const std::string leafType = sizeLeaf->GetTypeName();
+         if (leafType == "Int_t") {
+            fSizeReader.reset(new TSizeReaderImpl<Int_t>(*treeReader, foundLeafName.c_str()));
+         } else if (leafType == "UInt_t") {
+            fSizeReader.reset(new TSizeReaderImpl<UInt_t>(*treeReader, foundLeafName.c_str()));
+         } else if (leafType == "Short_t") {
+            fSizeReader.reset(new TSizeReaderImpl<Short_t>(*treeReader, foundLeafName.c_str()));
+         } else if (leafType == "UShort_t") {
+            fSizeReader.reset(new TSizeReaderImpl<UShort_t>(*treeReader, foundLeafName.c_str()));
+         } else if (leafType == "Long_t") {
+            fSizeReader.reset(new TSizeReaderImpl<Long_t>(*treeReader, foundLeafName.c_str()));
+         } else if (leafType == "ULong_t") {
+            fSizeReader.reset(new TSizeReaderImpl<ULong_t>(*treeReader, foundLeafName.c_str()));
+         } else if (leafType == "Long64_t") {
+            fSizeReader.reset(new TSizeReaderImpl<Long64_t>(*treeReader, foundLeafName.c_str()));
+         } else if (leafType == "ULong64_t") {
+            fSizeReader.reset(new TSizeReaderImpl<ULong64_t>(*treeReader, foundLeafName.c_str()));
          } else {
-            fSizeReader.reset(new TTreeReaderValue<Int_t>(*treeReader, foundLeafName.c_str()));
+            Error("TDynamicArrayReader ",
+                  "Unsupported size type for leaf %s. Supported types are int, short int, long int, long long int and "
+                  "their unsigned counterparts.",
+                  leafName);
          }
       }
 
-      size_t GetSize(ROOT::Detail::TBranchProxy* /*proxy*/) override {
-         if (fIsUnsigned)
-            return *GetSizeReader<UInt_t>();
-         return *GetSizeReader<Int_t>();
-      }
+      size_t GetSize(ROOT::Detail::TBranchProxy * /*proxy*/) override { return fSizeReader->GetSize(); }
    };
 
-   class TArrayParameterSizeReader: public TUIntOrIntReader<TObjectArrayReader> {
+   class TArrayParameterSizeReader : public TDynamicArrayReader<TObjectArrayReader> {
    public:
-      TArrayParameterSizeReader(TTreeReader *treeReader, const char *branchName):
-         TUIntOrIntReader<TObjectArrayReader>(treeReader, branchName) {}
+      TArrayParameterSizeReader(TTreeReader *treeReader, const char *branchName)
+         : TDynamicArrayReader<TObjectArrayReader>(treeReader, branchName)
+      {
+      }
    };
 
    // Reader interface for fixed size arrays
@@ -280,32 +303,32 @@ namespace {
    public:
       TArrayFixedSizeReader(Int_t sizeArg) : fSize(sizeArg) {}
 
-      virtual size_t GetSize(ROOT::Detail::TBranchProxy* /*proxy*/) { return fSize; }
+      size_t GetSize(ROOT::Detail::TBranchProxy* /*proxy*/) override { return fSize; }
    };
 
    class TBasicTypeArrayReader final: public TVirtualCollectionReader {
    public:
-      ~TBasicTypeArrayReader() {}
+      ~TBasicTypeArrayReader() override {}
 
       TVirtualCollectionProxy* GetCP (ROOT::Detail::TBranchProxy *proxy) {
          if (!proxy->Read()){
             fReadStatus = TTreeReaderValueBase::kReadError;
             Error("TBasicTypeArrayReader::GetCP()", "Read error in TBranchProxy.");
-            return 0;
+            return nullptr;
          }
          fReadStatus = TTreeReaderValueBase::kReadSuccess;
          return (TVirtualCollectionProxy*) proxy->GetCollection();
       }
 
-      virtual size_t GetSize(ROOT::Detail::TBranchProxy* proxy){
+      size_t GetSize(ROOT::Detail::TBranchProxy* proxy) override{
          TVirtualCollectionProxy *myCollectionProxy = GetCP(proxy);
          if (!myCollectionProxy) return 0;
          return myCollectionProxy->Size();
       }
 
-      virtual void* At(ROOT::Detail::TBranchProxy* proxy, size_t idx){
+      void* At(ROOT::Detail::TBranchProxy* proxy, size_t idx) override{
          TVirtualCollectionProxy *myCollectionProxy = GetCP(proxy);
-         if (!myCollectionProxy) return 0;
+         if (!myCollectionProxy) return nullptr;
          return (Byte_t*)myCollectionProxy->At(idx) + proxy->GetOffset();
       }
    };
@@ -316,9 +339,9 @@ namespace {
    public:
       TBasicTypeClonesReader(Int_t offsetArg) : fOffset(offsetArg) {}
 
-      virtual void* At(ROOT::Detail::TBranchProxy* proxy, size_t idx){
+      void* At(ROOT::Detail::TBranchProxy* proxy, size_t idx) override{
          TClonesArray *myClonesArray = GetCA(proxy);
-         if (!myClonesArray) return 0;
+         if (!myClonesArray) return nullptr;
          return (Byte_t*)myClonesArray->At(idx) + fOffset;
       }
    };
@@ -330,17 +353,17 @@ namespace {
    public:
       TLeafReader(TTreeReaderValueBase *valueReaderArg) : fValueReader(valueReaderArg), fElementSize(-1) {}
 
-      virtual size_t GetSize(ROOT::Detail::TBranchProxy* /*proxy*/){
+      size_t GetSize(ROOT::Detail::TBranchProxy* /*proxy*/) override{
          TLeaf *myLeaf = fValueReader->GetLeaf();
          return myLeaf ? myLeaf->GetLen() : 0; // Error will be printed by GetLeaf
       }
 
-      virtual void* At(ROOT::Detail::TBranchProxy* /*proxy*/, size_t idx){
+      void* At(ROOT::Detail::TBranchProxy* /*proxy*/, size_t idx) override{
          ProxyRead();
          void *address = fValueReader->GetAddress();
          if (fElementSize == -1){
             TLeaf *myLeaf = fValueReader->GetLeaf();
-            if (!myLeaf) return 0; // Error will be printed by GetLeaf
+            if (!myLeaf) return nullptr; // Error will be printed by GetLeaf
             fElementSize = myLeaf->GetLenType();
          }
          return (Byte_t*)address + (fElementSize * idx);
@@ -352,15 +375,16 @@ namespace {
       }
    };
 
-   class TLeafParameterSizeReader: public TUIntOrIntReader<TLeafReader> {
+   class TLeafParameterSizeReader : public TDynamicArrayReader<TLeafReader> {
    public:
-      TLeafParameterSizeReader(TTreeReader *treeReader, const char *leafName,
-                               TTreeReaderValueBase *valueReaderArg) :
-         TUIntOrIntReader<TLeafReader>(treeReader, leafName, valueReaderArg) {}
+      TLeafParameterSizeReader(TTreeReader *treeReader, const char *leafName, TTreeReaderValueBase *valueReaderArg)
+         : TDynamicArrayReader<TLeafReader>(treeReader, leafName, valueReaderArg)
+      {
+      }
 
       size_t GetSize(ROOT::Detail::TBranchProxy* proxy) override {
          ProxyRead();
-         return TUIntOrIntReader<TLeafReader>::GetSize(proxy);
+         return TDynamicArrayReader<TLeafReader>::GetSize(proxy);
       }
    };
 }
@@ -389,7 +413,7 @@ void ROOT::Internal::TTreeReaderArrayBase::CreateProxy()
       TBranch* br = fTreeReader->GetTree()->GetBranch(fBranchName);
       const char* brDataType = "{UNDETERMINED}";
       if (br) {
-         TDictionary* dictUnused = 0;
+         TDictionary* dictUnused = nullptr;
          brDataType = GetBranchDataType(br, dictUnused, fDict);
       }
       Error("TTreeReaderArrayBase::CreateProxy()", "The template argument type T of %s accessing branch %s (which contains data of type %s) is not known to ROOT. You will need to create a dictionary for it.",
@@ -403,7 +427,7 @@ void ROOT::Internal::TTreeReaderArrayBase::CreateProxy()
    // Search for the branchname, determine what it contains, and wire the
    // TBranchProxy representing it to us so we can access its data.
 
-   TDictionary* branchActualType = 0;
+   TDictionary* branchActualType = nullptr;
    TBranch* branch = nullptr;
    TLeaf *myLeaf = nullptr;
    if (!GetBranchAndLeaf(branch, myLeaf, branchActualType))
@@ -492,7 +516,7 @@ void ROOT::Internal::TTreeReaderArrayBase::CreateProxy()
                fBranchName.Data(), nonCollTypeName, nonCollTypeName);
          if (fSetupStatus == kSetupInternalError)
             fSetupStatus = kSetupNotACollection;
-         fProxy = 0;
+         fProxy = nullptr;
          return;
       }
       if (!branchActualType) {
@@ -505,7 +529,7 @@ void ROOT::Internal::TTreeReaderArrayBase::CreateProxy()
             if (fSetupStatus == kSetupInternalError)
                fSetupStatus = kSetupMissingDictionary;
          }
-         fProxy = 0;
+         fProxy = nullptr;
          return;
       }
 
@@ -516,6 +540,12 @@ void ROOT::Internal::TTreeReaderArrayBase::CreateProxy()
             return false;
          auto left_datatype = dynamic_cast<TDataType *>(left);
          auto right_datatype = dynamic_cast<TDataType *>(right);
+         auto left_enum = dynamic_cast<TEnum*>(left);
+         auto right_enum = dynamic_cast<TEnum*>(right);
+
+         if ((left_datatype && left_datatype->GetType() == kInt_t && right_enum)
+            || (right_datatype && right_datatype->GetType() == kInt_t && left_enum))
+            return true;
          if (!left_datatype || !right_datatype)
             return false;
          auto l = left_datatype->GetType();
@@ -561,7 +591,7 @@ bool ROOT::Internal::TTreeReaderArrayBase::GetBranchAndLeaf(TBranch* &branch, TL
    if (!fBranchName.Contains(".")) {
       Error("TTreeReaderArrayBase::GetBranchAndLeaf()", "The tree does not have a branch called %s. You could check with TTree::Print() for available branches.", fBranchName.Data());
       fSetupStatus = kSetupMissingBranch;
-      fProxy = 0;
+      fProxy = nullptr;
       return false;
    }
 
@@ -572,7 +602,7 @@ bool ROOT::Internal::TTreeReaderArrayBase::GetBranchAndLeaf(TBranch* &branch, TL
    if (!branch){
       Error("TTreeReaderArrayBase::GetBranchAndLeaf()", "The tree does not have a branch called %s. You could check with TTree::Print() for available branches.", fBranchName.Data());
       fSetupStatus = kSetupMissingBranch;
-      fProxy = 0;
+      fProxy = nullptr;
       return false;
    }
 
@@ -580,7 +610,7 @@ bool ROOT::Internal::TTreeReaderArrayBase::GetBranchAndLeaf(TBranch* &branch, TL
    if (!myLeaf){
       Error("TTreeReaderArrayBase::GetBranchAndLeaf()", "The tree does not have a branch, nor a sub-branch called %s. You could check with TTree::Print() for available branches.", fBranchName.Data());
       fSetupStatus = kSetupMissingBranch;
-      fProxy = 0;
+      fProxy = nullptr;
       return false;
    }
 
@@ -588,7 +618,7 @@ bool ROOT::Internal::TTreeReaderArrayBase::GetBranchAndLeaf(TBranch* &branch, TL
    if (!tempDict){
       Error("TTreeReaderArrayBase::GetBranchAndLeaf()", "Failed to get the dictionary for %s.", myLeaf->GetTypeName());
       fSetupStatus = kSetupMissingDictionary;
-      fProxy = 0;
+      fProxy = nullptr;
       return false;
    }
 
@@ -603,7 +633,7 @@ bool ROOT::Internal::TTreeReaderArrayBase::GetBranchAndLeaf(TBranch* &branch, TL
    }
    else {
       Error("TTreeReaderArrayBase::GetBranchAndLeaf()", "Leaf of type %s cannot be read by TTreeReaderValue<%s>.", myLeaf->GetTypeName(), fDict->GetName());
-      fProxy = 0;
+      fProxy = nullptr;
       fSetupStatus = kSetupMismatch;
       return false;
    }
@@ -649,12 +679,18 @@ void ROOT::Internal::TTreeReaderArrayBase::SetImpl(TBranch* branch, TLeaf* myLea
       if (id >= 0){ // Not root node?
          // Int_t offset = streamerInfo->GetOffsets()[id];
          TStreamerElement *element = (TStreamerElement*)streamerInfo->GetElements()->At(id);
-         // Bool_t isPointer = element->IsaPointer();
+         // bool isPointer = element->IsaPointer();
          // TClass *classPointer = element->GetClassPointer();
 
          if (fSetupStatus == kSetupInternalError)
             fSetupStatus = kSetupMatch;
-         if (element->IsA() == TStreamerSTL::Class()){
+         if (element->IsA() == TStreamerSTL::Class()) {
+            if (branchElement->GetType() == 31) {
+               Error("TTreeReaderArrayBase::SetImpl",
+                     "STL Collection nested in a TClonesArray not yet supported");
+               fSetupStatus = kSetupInternalError;
+               return;
+            }
             fImpl = std::make_unique<TSTLReader>();
          }
          else if (element->IsA() == TStreamerObject::Class()){
@@ -684,7 +720,10 @@ void ROOT::Internal::TTreeReaderArrayBase::SetImpl(TBranch* branch, TLeaf* myLea
             else if (branchElement->GetType() == TBranchElement::kClonesMemberNode){
                fImpl = std::make_unique<TBasicTypeClonesReader>(element->GetOffset());
             }
-            else {
+            else if (fDict->IsA() == TEnum::Class()) {
+               fImpl = std::make_unique<TArrayFixedSizeReader>(element->GetArrayLength());
+               ((TObjectArrayReader*)fImpl.get())->SetBasicTypeSize(sizeof(Int_t));
+            } else {
                fImpl = std::make_unique<TArrayFixedSizeReader>(element->GetArrayLength());
                ((TObjectArrayReader*)fImpl.get())->SetBasicTypeSize(((TDataType*)fDict)->Size());
             }
@@ -766,7 +805,7 @@ const char* ROOT::Internal::TTreeReaderArrayBase::GetBranchContentDataType(TBran
             TClass *myClass = collProxy->GetValueClass();
             if (!myClass){
                Error("TTreeReaderArrayBase::GetBranchContentDataType()", "Could not get value class.");
-               return 0;
+               return nullptr;
             }
             dict = TDictionary::GetDictionary(myClass->GetName());
             if (!dict) dict = TDataType::GetDataType(collProxy->GetType());
@@ -777,7 +816,7 @@ const char* ROOT::Internal::TTreeReaderArrayBase::GetBranchContentDataType(TBran
             if (brElement->GetType() == 3) {
                contentTypeName = brElement->GetClonesName();
                dict = TDictionary::GetDictionary(brElement->GetClonesName());
-               return 0;
+               return nullptr;
             }
             // STL:
             TClassEdit::TSplitType splitType(brElement->GetClassName());
@@ -794,13 +833,13 @@ const char* ROOT::Internal::TTreeReaderArrayBase::GetBranchContentDataType(TBran
                contentTypeName += splitType.fElements[2];
                contentTypeName += " >";
             }
-            return 0;
+            return nullptr;
          }
-         return 0;
+         return nullptr;
       } else if (brElement->GetType() == 31
                  || brElement->GetType() == 41) {
          // it's a member, extract from GetClass()'s streamer info
-         TClass* clData = 0;
+         TClass* clData = nullptr;
          EDataType dtData = kOther_t;
          int ExpectedTypeRet = brElement->GetExpectedType(clData, dtData);
          if (ExpectedTypeRet == 0) {
@@ -818,9 +857,9 @@ const char* ROOT::Internal::TTreeReaderArrayBase::GetBranchContentDataType(TBran
                Error("TTreeReaderArrayBase::GetBranchContentDataType()", "The branch %s contains a data type %d for which the dictionary cannot be retrieved.",
                      branch->GetName(), (int)dtData);
                contentTypeName = TDataType::GetTypeName(dtData);
-               return 0;
+               return nullptr;
             }
-            return 0;
+            return nullptr;
          } else if (ExpectedTypeRet == 1) {
             int brID = brElement->GetID();
             if (brID == -1) {
@@ -828,14 +867,14 @@ const char* ROOT::Internal::TTreeReaderArrayBase::GetBranchContentDataType(TBran
                Error("TTreeReaderArrayBase::GetBranchContentDataType()", "The branch %s contains data of type %s for which the dictionary does not exist. It's needed.",
                      branch->GetName(), brElement->GetClassName());
                contentTypeName = brElement->GetClassName();
-               return 0;
+               return nullptr;
             }
             // Either the data type name doesn't have an EDataType entry
             // or the streamer info doesn't have a TClass* attached.
             TStreamerElement* element =
                (TStreamerElement*) brElement->GetInfo()->GetElement(brID);
             contentTypeName = element->GetTypeName();
-            return 0;
+            return nullptr;
          }
          /* else (ExpectedTypeRet == 2)*/
          // The streamer info entry cannot be found.
@@ -853,12 +892,12 @@ const char* ROOT::Internal::TTreeReaderArrayBase::GetBranchContentDataType(TBran
                TClass *myClass = brElement->GetCurrentClass();
                if (!myClass){
                   Error("TTreeReaderArrayBase::GetBranchDataType()", "Could not get class from branch element.");
-                  return 0;
+                  return nullptr;
                }
                TVirtualCollectionProxy *myCollectionProxy = myClass->GetCollectionProxy();
                if (!myCollectionProxy){
                   Error("TTreeReaderArrayBase::GetBranchDataType()", "Could not get collection proxy from STL class");
-                  return 0;
+                  return nullptr;
                }
                // Try getting the contained class
                dict = myCollectionProxy->GetValueClass();
@@ -866,22 +905,22 @@ const char* ROOT::Internal::TTreeReaderArrayBase::GetBranchContentDataType(TBran
                if (!dict) dict = TDataType::GetDataType(myCollectionProxy->GetType());
                if (!dict){
                   Error("TTreeReaderArrayBase::GetBranchDataType()", "Could not get valueClass from collectionProxy.");
-                  return 0;
+                  return nullptr;
                }
                contentTypeName = dict->GetName();
-               return 0;
+               return nullptr;
             }
             else if (element->IsA() == TStreamerObject::Class() && !strcmp(element->GetTypeName(), "TClonesArray")){
                if (!fProxy->Setup() || !fProxy->Read()){
                   Error("TTreeReaderArrayBase::GetBranchContentDataType()", "Failed to get type from proxy, unable to check type");
                   contentTypeName = "UNKNOWN";
-                  dict = 0;
+                  dict = nullptr;
                   return contentTypeName;
                }
                TClonesArray *myArray = (TClonesArray*)fProxy->GetWhere();
                dict = myArray->GetClass();
                contentTypeName = dict->GetName();
-               return 0;
+               return nullptr;
             }
             else {
                dict = brElement->GetCurrentClass();
@@ -890,7 +929,7 @@ const char* ROOT::Internal::TTreeReaderArrayBase::GetBranchContentDataType(TBran
                   dict = TDataType::GetDataType((EDataType)((TDataType*)myDataType)->GetType());
                }
                contentTypeName = brElement->GetTypeName();
-               return 0;
+               return nullptr;
             }
          }
          if (brElement->GetCurrentClass() == TClonesArray::Class()){
@@ -905,17 +944,17 @@ const char* ROOT::Internal::TTreeReaderArrayBase::GetBranchContentDataType(TBran
             // If it fails, try to get the contained type as a primitive type
             if (!dict) dict = TDataType::GetDataType(brElement->GetClass()->GetCollectionProxy()->GetType());
             if (dict) contentTypeName = dict->GetName();
-            return 0;
+            return nullptr;
          }
          else if (!dict){
             dict = brElement->GetClass();
             contentTypeName = dict->GetName();
-            return 0;
+            return nullptr;
          }
 
-         return 0;
+         return nullptr;
       }
-      return 0;
+      return nullptr;
    } else if (branch->IsA() == TBranch::Class()
               || branch->IsA() == TBranchObject::Class()
               || branch->IsA() == TBranchSTL::Class()) {
@@ -933,25 +972,25 @@ const char* ROOT::Internal::TTreeReaderArrayBase::GetBranchContentDataType(TBran
                else if (typeEnumConstant == kFloat16_t) typeEnumConstant = kFloat_t;
                dict = TDataType::GetDataType(typeEnumConstant);
                contentTypeName = myLeaf->GetTypeName();
-               return 0;
+               return nullptr;
             }
          }
 
          // leaflist. Can't represent.
          Error("TTreeReaderArrayBase::GetBranchContentDataType()", "The branch %s was created using a leaf list and cannot be represented as a C++ type. Please access one of its siblings using a TTreeReaderArray:", branch->GetName());
          TIter iLeaves(branch->GetListOfLeaves());
-         TLeaf* leaf = 0;
+         TLeaf* leaf = nullptr;
          while ((leaf = (TLeaf*) iLeaves())) {
             Error("TTreeReaderArrayBase::GetBranchContentDataType()", "   %s.%s", branch->GetName(), leaf->GetName());
          }
-         return 0;
+         return nullptr;
       }
       if (dataTypeName) dict = TDictionary::GetDictionary(dataTypeName);
       if (branch->IsA() == TBranchSTL::Class()){
          Warning("TTreeReaderArrayBase::GetBranchContentDataType()", "Not able to check type correctness, ignoring check");
          dict = fDict;
          fSetupStatus = kSetupNoCheck;
-         return 0;
+         return nullptr;
       }
       return dataTypeName;
    } else if (branch->IsA() == TBranchClones::Class()) {
@@ -960,11 +999,11 @@ const char* ROOT::Internal::TTreeReaderArrayBase::GetBranchContentDataType(TBran
    } else if (branch->IsA() == TBranchRef::Class()) {
       // Can't represent.
       Error("TTreeReaderArrayBase::GetBranchContentDataType()", "The branch %s is a TBranchRef and cannot be represented as a C++ type.", branch->GetName());
-      return 0;
+      return nullptr;
    } else {
       Error("TTreeReaderArrayBase::GetBranchContentDataType()", "The branch %s is of type %s - something that is not handled yet.", branch->GetName(), branch->IsA()->GetName());
-      return 0;
+      return nullptr;
    }
 
-   return 0;
+   return nullptr;
 }
